/* RC5REF.C -- Reference implementation of RC5-32/16/64 in C.                   */
/* Copyright (C) 1995 RSA Data Security, Inc.                                   */
/* Edited by Alexander Myasnikov                                                */
/* Web: www.darksoftware.narod.ru                                               */



#include <stdlib.h>

typedef unsigned long int u32;          /* Should be 32-bit = 4 bytes        */
typedef unsigned char u8;               /* Should be 8-bit = 4 bytes         */

#define w        32                     /* u32 size in bits                  */
#define r        16                     /* number of rounds                  */
#define b        64                     /* number of bytes in key            */
#define c        16                     /* number u32s in key = ceil(8*b/w)  */
#define t        34                     /* size of table S = 2*(r+1) u32s    */

u32 S[t];                               /* expanded key table                */

u32 P = 0xb7e15163, Q = 0x9e3779b9;     /* magic constants                   */


#define ROTL(x,y) (_lrotl(x,y))
#define ROTR(x,y) (_lrotr(x,y))

void __stdcall __export
crypt (u32 * pt)
{
  u32 i, A = pt[0] + S[0], B = pt[1] + S[1];
  for (i = 1; i <= r; i++)
    {
      A = ROTL (A ^ B, B) + S[2 * i];
      B = ROTL (B ^ A, A) + S[2 * i + 1];
    }
  pt[0] = A;
  pt[1] = B;
}

void __stdcall __export
decrypt (u32 * ct)
{
  u32 i, B = ct[1], A = ct[0];
  for (i = r; i > 0; i--)
    {
      B = ROTR (B - S[2 * i + 1], A) ^ A;
      A = ROTR (A - S[2 * i], B) ^ B;
    }
  ct[1] = B - S[1];
  ct[0] = A - S[0];
}

void __stdcall __export
setup (u8 * K)
{
  u32 i, j, k, u = w / 8, A, B, L[c];

  for (i = b - 1, L[c - 1] = 0; i != -1; i--)
    L[i / u] = (L[i / u] << 8) + K[i];
  for (S[0] = P, i = 1; i < t; i++)
    S[i] = S[i - 1] + Q;
  for (A = B = i = j = k = 0; k < 3 * t; k++, i = (i + 1) % t, j = (j + 1) % c) /* 3*t > 3*c */
    {
      A = S[i] = ROTL (S[i] + (A + B), 3);
      B = L[j] = ROTL (L[j] + (A + B), (A + B));
    }
}
